﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TransportGuideContracts.BindingModels;
using TransportGuideContracts.BusinessLogicsContracts;

namespace TransportGuide
{
	public partial class FormRoutes : Form
	{
		private readonly ILogger _logger;
		private readonly IRouteLogic _logic;

		public FormRoutes(ILogger<FormRoutes> logger, IRouteLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
		}

		private void LoadData()
		{
			try
			{
				var list = _logic.ReadList(null);

				if (list != null)
				{
					DataGridView.DataSource = list;
					DataGridView.Columns["Id"].Visible = false;
					DataGridView.Columns["Name"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
					DataGridView.Columns["StopRoutes"].Visible = false;

				}

				_logger.LogInformation("Загрузка компонентов");

			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки компонентов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void AddButton_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormRoute));

			if (service is FormRoute form)
			{
				if (form.ShowDialog() == DialogResult.OK)
				{
					LoadData();
				}
			}
		}
		private void ChangeButton_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				var service = Program.ServiceProvider?.GetService(typeof(FormRoute));

				if (service is FormRoute form)
				{
					form.Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);

					if (form.ShowDialog() == DialogResult.OK)
					{
						LoadData();
					}
				}
			}
		}
		private void DeleteButton_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);

					try
					{
						if (!_logic.Delete(new RouteBindingModel
						{
							Id = id
						}))
						{
							throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
						}

						LoadData();
					}
					catch (Exception ex)
					{
						_logger.LogError(ex, "Ошибка удаления компонента");
						MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
		}
		private void UpdateButton_Click(object sender, EventArgs e)
		{
			LoadData();
		}


		private void FormRoutes_Load(object sender, EventArgs e)
		{
			LoadData();
		}
	}
}
