﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TransportGuideContracts.BindingModels;
using TransportGuideContracts.BusinessLogicsContracts;

namespace TransportGuide
{
	public partial class FormStops : Form
	{
		private readonly ILogger _logger;
		private readonly IStopLogic _logic;
		public FormStops(ILogger<FormStops> logger, IStopLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
		}

		private void FormStops_Load(object sender, EventArgs e)
		{
			LoadData();
		}
		private void LoadData()
		{
			try
			{
				var list = _logic.ReadList(null);
				if (list != null)
				{
					dataGridView.DataSource = list;
					dataGridView.Columns["Id"].Visible = false;
					dataGridView.Columns["Name"].AutoSizeMode =
				   DataGridViewAutoSizeColumnMode.Fill;
				}
				_logger.LogInformation("Загрузка компонентов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки компонентов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
			   MessageBoxIcon.Error);
			}
		}

		private void buttonAdd_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormStop));
			if (service is FormStop form)
			{
				if (form.ShowDialog() == DialogResult.OK)
				{
					LoadData();
				}
			}
		}

		private void buttonUpdate_Click(object sender, EventArgs e)
		{
			LoadData();
		}

		private void buttonDelete_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				if (MessageBox.Show("Удалить запись?", "Вопрос",
			   MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					int id =
				   Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
					_logger.LogInformation("Удаление компонента");
					try
					{
						if (!_logic.Delete(new StopBindingModel
						{
							Id = id
						}))
						{
							throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
						}
						LoadData();
					}
					catch (Exception ex)
					{
						_logger.LogError(ex, "Ошибка удаления компонента");
						MessageBox.Show(ex.Message, "Ошибка",
					   MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}

		}

		private void buttonChange_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				var service =
			   Program.ServiceProvider?.GetService(typeof(FormStop));
				if (service is FormStop form)
				{
					form.Id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
					if (form.ShowDialog() == DialogResult.OK)
					{
						LoadData();
					}
				}
			}
		}
	}
}
