﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TransportGuideContracts.BindingModels;
using TransportGuideContracts.BusinessLogicsContracts;
using TransportGuideContracts.SearchModels;

namespace TransportGuide
{
	public partial class FormTransportType : Form
	{
		private readonly ITransportTypeLogic _logic;
		private int? _id;
		public int Id { set { _id = value; } }
		public FormTransportType(ITransportTypeLogic logic)
		{
			InitializeComponent();
			_logic = logic;
		}
		private void FormTransportType_Load(object sender, EventArgs e)
		{
			if (_id.HasValue)
			{
				try
				{
					var view = _logic.ReadElement(new TransportTypeSearchModel
					{
						Id = _id.Value
					});
					if (view != null)
					{
						textBoxName.Text = view.Name;
						textBoxPrice.Text = view.Price.ToString();
					}
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message, "Loading error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				}
			}
		}
		private void buttonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxName.Text))
			{
				MessageBox.Show("Error", "Name cant be empty",
			   MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			try
			{
				var model = new TransportTypeBindingModel
				{
					Id = _id ?? 0,
					Name = textBoxName.Text,
					Price = Convert.ToDouble(textBoxPrice.Text)
				};
				var operationResult = _id.HasValue ? _logic.Update(model) :
			   _logic.Create(model);
				if (!operationResult)
				{
					throw new Exception("Error with save.");
				}
				MessageBox.Show("Transport Type was saved", "", 
			   MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK,
			   MessageBoxIcon.Error);
			}
		}
		private void buttonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}

		
	}
}
